// this file is covered by the General Public License version 2 or later
// please see GPL.html for more details and licensing issues
// copyright Etienne de Foras ( the author )  mailto: etienne.deforas@gmail.com

#ifndef _Device_
#define _Device_

#include <string>
#include <vector>
using namespace std;

class Surface;
class Light;

class Device
{
public:
    Device();
    virtual ~Device();

    void insert_surface(string sGeometry,string sKind,int iPos);
    void delete_surface(int iPos);
    bool set_kind(int iSurf,string sKind);
    string get_kind(int iSurf) const;
    bool set_geometry(int iSurf,string sGeometry);
    string get_geometry(int iSurf) const;
    int nb_surface() const;

    void set_light_tilt(double dTiltX,double dTiltY);
    void get_light_tilt(double& dTiltX,double& dTiltY) const;
    void set_light_colors(const string & _sLightColors);
    void get_light_colors(string &_sLightColors) const;
    void set_light_grid(int iGridX,int iGridY);
    Light& get_light_at(int iSurface);

    bool get(int iSurf,string sParam,double& dVal);
    bool set(int iSurf,string sParam,double dVal);

    double get_profile_at(int iSurface,double dX,double dY);

    void set_autofocus(double dAutofocusZMin,double dAutofocusZMax,double dAutofocusZStep);
    void get_autofocus(double& dAutofocusZMin,double& dAutofocusZMax,double& dAutofocusZStep);

    void enable_auto_diameter(bool bEnable);

private:
    void initialise_light();
    void receive(Light& l,int iSurface);
    void ray_trace(int iSurfaceMax);

    vector<Surface*> _theSurf;

    Light* _pLight;
    double _dTiltX,_dTiltY;
    int _iGridX;
    int _iGridY;
    string _sLightColors;

    double _dAutofocusZMin;
    double _dAutofocusZMax;
    double _dAutofocusZStep;
};

#endif
