// this file is covered by the General Public License version 2 or later
// please see GPL.html for more details and licensing issues
// copyright Etienne de Foras ( the author )  mailto: etienne.deforas@gmail.com

#include <sstream>
using namespace std;

#include "DeviceIo.h"
#include "Device.h"

//#include "Light.h"
#include "Properties.h"

////////////////////////////////////////////////////////////////////////////////
bool DeviceIo::save(string sFile,Device* pD)
{
    Properties ac;

    for (int iS=0;iS<pD->nb_surface();iS++)
    {
        stringstream ss;
        string sSurfName;
        ss << iS;
        ss >> sSurfName;

        ac.set(sSurfName+".geometry",pD->get_geometry(iS));
        ac.set(sSurfName+".kind",pD->get_kind(iS));

        double dZ;
        pD->get(iS,"z",dZ);
        ac.set(sSurfName+".z",dZ);

        double dConic;
        if (pD->get(iS,"conic",dConic))
            ac.set(sSurfName+".conic",dConic);

        double dDiameter;
        pD->get(iS,"diameter",dDiameter);
        ac.set(sSurfName+".diameter",dDiameter);

        double dRC;
        pD->get(iS,"radius_curvature",dRC);
        ac.set(sSurfName+".radius_curvature",dRC);
    }

    //write the light properties (if diff form zero)
    double dTiltX=0. , dTiltY=0.;
    pD->get_light_tilt(dTiltX,dTiltY);
    if (dTiltX!=0.)
        ac.set("light.tilt_x",dTiltX);
    if (dTiltY!=0.)
        ac.set("light.tilt_y",dTiltY);

    string sLightColors;
    pD->get_light_colors(sLightColors);
    ac.set("light.colors",sLightColors);

    // sauve les prop autofocus
    double dAutofocusZMin=0;
    double dAutofocusZMax=0;
    double dAutofocusZStep=0;
    pD->get_autofocus(dAutofocusZMin,dAutofocusZMax,dAutofocusZStep);
    ac.set("autofocus_z_min",dAutofocusZMin);
    ac.set("autofocus_z_max",dAutofocusZMax);
    ac.set("autofocus_z_step",dAutofocusZStep);

    ac.save(sFile);
    return true;
}
////////////////////////////////////////////////////////////////////////////////
Device* DeviceIo::load(string sFile)
{
    Properties ac;
    if (!ac.load(sFile))
        return 0;

    Device* pD=new Device();

    int iS=0;
    string sSurfName="0";
    do
    {
        if (ac.exist(sSurfName+".geometry")==false)
            break;

        if (ac.exist(sSurfName+".kind")==false)
            break;

        string sGeometry=ac.get(sSurfName+".geometry");
        string sKind=ac.get(sSurfName+".kind");
        pD->insert_surface(sGeometry,sKind,iS);

        if (ac.exist(sSurfName+".diameter"))
        {
            double dDiameter=ac.get_double(sSurfName+".diameter");
            pD->set(iS,"diameter",dDiameter);
        }

        if (ac.exist(sSurfName+".radius_curvature"))
        {
            double dRadius=ac.get_double(sSurfName+".radius_curvature");
            pD->set(iS,"radius_curvature",dRadius);
        }

        if (ac.exist(sSurfName+".conic"))
        {
            double dConic=ac.get_double(sSurfName+".conic");
            pD->set(iS,"conic",dConic);
        }

        if (ac.exist(sSurfName+".z"))
        {
            double dRadius=ac.get_double(sSurfName+".z");
            pD->set(iS,"z",dRadius);
        }

        iS++;
        stringstream ss2;
        ss2 << iS;
        ss2 >> sSurfName;
    }
    while (ac.exist(sSurfName+".kind"));

    //read the light properties
    double dTiltX=0. , dTiltY=0.;

    if (ac.exist("light.tilt_x"))
        dTiltX=ac.get_double("light.tilt_x");

    if (ac.exist("light.tilt_y"))
        dTiltY=ac.get_double("light.tilt_y");

    pD->set_light_tilt(dTiltX,dTiltY);

    if (ac.exist("light.colors"))
    {
        pD->set_light_colors(ac.get("light.colors"));
    }
    else
        pD->set_light_colors(""); //temporaire

    // lit les prop autofocus
    double dAutofocusZMin=ac.get_double("autofocus_z_min");
    double dAutofocusZMax=ac.get_double("autofocus_z_max");
    double dAutofocusZStep=ac.get_double("autofocus_z_step");
    pD->set_autofocus(dAutofocusZMin,dAutofocusZMax,dAutofocusZStep);

    return pD;
}
////////////////////////////////////////////////////////////////////////////////
