// this file is covered by the General Public License version 2 or later
// please see GPL.html for more details and licensing issues
// copyright Etienne de Foras ( the author )  mailto: etienne.deforas@gmail.com

#include "DeviceModifierAF.h"
#include "Device.h"
#include "Light.h"

#include <cassert>

////////////////////////////////////////////////////////////////////////////////
bool DeviceModifierAF::calc(Device* pDevice)
{
    assert(pDevice!=0);

    double dZmin=0;
    double dZmax=0;
    double dZStep=0;
    pDevice->get_autofocus(dZmin,dZmax,dZStep);

    int iMovingSurface=pDevice->nb_surface()-1;

    double dA=dZmin,dC=dZmax;
    double dB=(dA+dC)/2.,dAB,dBC;

    double dQA,dQB,dQC,dQAB,dQBC;

    pDevice->set(iMovingSurface,"z",dA);
    dQA=pDevice->get_light_at(iMovingSurface).get_psf_diameter();

    pDevice->set(iMovingSurface,"z",dB);
    dQB=pDevice->get_light_at(iMovingSurface).get_psf_diameter();

    pDevice->set(iMovingSurface,"z",dC);
    dQC=pDevice->get_light_at(iMovingSurface).get_psf_diameter();

    if (dQA==-1.) return false;
    if (dQB==-1.) return false;
    if (dQC==-1.) return false;

    while (dC-dA>dZStep)
    {
        dAB=(dA+dB)/2.;
        pDevice->set(iMovingSurface,"z",dAB);
        dQAB=pDevice->get_light_at(iMovingSurface).get_psf_diameter();

        if (dQAB<dQB)
        {
            // un nouveau min !! on recentre.
            dC=dB; //dQC=dQB;
            dB=dAB;
            dQB=dQAB;
        }
        else
        {
            dBC=(dB+dC)/2.;
            pDevice->set(iMovingSurface,"z",dBC);
            dQBC=pDevice->get_light_at(iMovingSurface).get_psf_diameter();

            if (dQBC<dQB)
            {
                // un nouveau min !! on recentre.
                dA=dB; //dQA=dQB;
                dB=dBC;
                dQB=dQBC;
            }
            else
            {
                // on recentre autour de B
                dA=dAB; //dQA=dQAB;
                dC=dBC; //dQC=dQBC;
            }
        }
    }

    pDevice->set(iMovingSurface,"z",dB);
    return true;
}
//////////////////////////////////////////////////////////////////////////////
