// this file is covered by the General Public License version 2 or later
// please see GPL.html for more details and licensing issues
// copyright Etienne de Foras ( the author )  mailto: etienne.deforas@gmail.com


#include "DockAutofocus.h"
#include "ui_DockAutofocus.h"

#include "Device.h"
#include "DeviceModifierAF.h"

DockAutofocus::DockAutofocus(QWidget *parent) :
        QDockWidget(parent),
        m_ui(new Ui::DockAutofocus)
{
    m_ui->setupUi(this);

    connect(m_ui->btnAutoFocus,SIGNAL(clicked()),this,SLOT(OnAF()));
}

DockAutofocus::~DockAutofocus()
{
    delete m_ui;
}

void DockAutofocus::changeEvent(QEvent *e)
{
    QDockWidget::changeEvent(e);
    switch (e->type())
    {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void DockAutofocus::device_changed(Device* pDevice)
{
    _pDevice=pDevice;
    double dZMin=0;
    double dZMax=0;
    double dZStep=0;
    _pDevice->get_autofocus(dZMin,dZMax,dZStep);

    m_ui->leZmin->setText(QString::number(dZMin));
    m_ui->leZmax->setText(QString::number(dZMax));
    m_ui->leZstep->setText(QString::number(dZStep));
}

void DockAutofocus::OnAF()
{
    double dZmin=m_ui->leZmin->text().toDouble();
    double dZmax=m_ui->leZmax->text().toDouble();
    double dZStep=m_ui->leZstep->text().toDouble();
    _pDevice->set_autofocus(dZmin,dZmax,dZStep);

    if (_pDevice->nb_surface()==0)
        return;

    _pDevice->enable_auto_diameter(true);
    DeviceModifierAF dm;
    dm.calc(_pDevice);

    emit device_changed(this);
}
