// this file is covered by the General Public License version 2 or later
// please see GPL.html for more details and licensing issues
// copyright Etienne de Foras ( the author )  mailto: etienne.deforas@gmail.com

#include "DockScatterPlot.h"
#include "ui_DockScatterPlot.h"

#include <QtGui>

#include "Device.h"
#include "Light.h"
#include "Photon.h"

#include <cassert>

void DockScatterPlot::enlarge(QRectF& r,double dRatio)
{
    double dX=r.width();
    double dY=r.height();
    QPointF c=r.center();
    r.setCoords( c.x()-dX*dRatio/2., c.y()-dY*dRatio/2., c.x()+dX*dRatio/2., c.y()+dY*dRatio/2.);
}

class myScene : public QGraphicsScene
{
protected:
    virtual void wheelEvent ( QGraphicsSceneWheelEvent * wheelEvent )
    {
        QGraphicsView* v=views()[0];
        v->setTransformationAnchor(QGraphicsView::AnchorUnderMouse);

        if (wheelEvent->delta()>0)
            v->scale(1.25,1.25);
        else
            v->scale(0.8,0.8);

        wheelEvent->accept();
    }
};


class SimpleItem : public QGraphicsItem
{
public:
    SimpleItem(int iColor):QGraphicsItem()
    {
        _iColor=iColor;
    }

    QRectF boundingRect() const
    {
        return QRectF(0,0,0,0);
    }

    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
    {
        painter->setRenderHint(QPainter::Antialiasing,false);
        painter->setPen(QColor(QRgb(_iColor)));
        painter->drawPoint(0,0);

    }
    int _iColor;
};

DockScatterPlot::DockScatterPlot(QWidget *parent) :
        QDockWidget(parent),
        m_ui(new Ui::DockScatterPlot)
{
    m_ui->setupUi(this);
    m_ui->graphicsView->setScene(new myScene); //TODO deleter la scene?

    m_ui->graphicsView->setDragMode (QGraphicsView::ScrollHandDrag);
    m_ui->graphicsView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    m_ui->graphicsView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    m_ui->graphicsView->setRenderHints( QPainter::Antialiasing );
}

DockScatterPlot::~DockScatterPlot()
{
    delete m_ui;
}

void DockScatterPlot::changeEvent(QEvent *e)
{
    QDockWidget::changeEvent(e);
    switch (e->type())
    {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void DockScatterPlot::device_changed(Device* pDevice)
{
    _pDevice=pDevice;
    QGraphicsScene* scene=m_ui->graphicsView->scene();
    scene->clear();

    if (pDevice->nb_surface()==0)
    {
        m_ui->qlPsfDiameter->setText("-1.");
        m_ui->qlFD->setText("-1.");
        m_ui->qlPerfectAiry->setText("-1.");
        m_ui->qlLFro->setText("-1.");
        return;
    }

    qDebug("DockScatterPlot::device_changed and raytrace");
    _pDevice->enable_auto_diameter(true);
    _pDevice->set_light_grid(30,30);
    Light& rLight=pDevice->get_light_at(pDevice->nb_surface()-1);

    double dCenterX,dCenterY;
    double dPsfDiameter=rLight.get_psf_diameter();
    double dFD=rLight.get_FD();
    double dPerfectAiryRadius=rLight.get_theorical_ro();
    double dLfro=rLight.get_lfro();
    //    rLight.get_psf_center(dCenterX,dCenterY);
    rLight.get_psf_center(dCenterY,dCenterX);
    dCenterX=-dCenterX;

    int iNbPhoton=rLight.nb_photon();
    for (int i=0;i<iNbPhoton;i++)
    {
        Photon& rP=rLight.get_photon(i);
        if (rP.valid==true)
        {
            double dX=rP.x;
            double dY=rP.y;

            assert(!isnan(dX));
            assert(!isnan(dY));

            QGraphicsItem * pi=new SimpleItem(rP.iVisualColor);
            pi->setPos(-dY,dX);
            pi->setZValue(10);
            scene->addItem(pi);
        }
    }

    QGraphicsItem * pEllipse=scene->addEllipse((dCenterX-dPerfectAiryRadius),(dCenterY-dPerfectAiryRadius),dPerfectAiryRadius*2.,dPerfectAiryRadius*2.);
    pEllipse->setZValue(20);

    // defini l'englobant des SimpleItem
    scene->addRect(dCenterX-dPsfDiameter/2.,dCenterY-dPsfDiameter/2.,dPsfDiameter,dPsfDiameter,QPen(Qt::NoPen));

    QRectF r=scene->itemsBoundingRect();
    enlarge(r,1.1);
    scene->setSceneRect(r);

    m_ui->graphicsView->fitInView(r,Qt::KeepAspectRatio);

    m_ui->qlPsfDiameter->setText(QString("%1").arg(dPsfDiameter*1000.));
    m_ui->qlFD->setText(QString("%1").arg(dFD));

    m_ui->qlPerfectAiry->setText(QString("%1").arg(dPerfectAiryRadius*2.*1000.));
    m_ui->qlLFro->setText(QString("%1").arg(dLfro));
}
