// this file is covered by the General Public License version 2 or later
// please see GPL.html for more details and licensing issues
// copyright Etienne de Foras ( the author )  mailto: etienne.deforas@gmail.com

#include "DockSurfacesData.h"
#include "ui_DockSurfacesData.h"

#include <QComboBox>
#include <cassert>
#include "Device.h"

DockSurfacesData::DockSurfacesData(QWidget *parent) :
        QDockWidget(parent),
        m_ui(new Ui::DockSurfacesData)
{
    m_ui->setupUi(this);

    QStringList qsl;
    qsl+="Id";
    qsl+="Geometry";
    qsl+="Kind";
    qsl+="RCurv";
    qsl+="Conic";
    qsl+="Z";
    qsl+="Diameter";

    m_ui->twSurfacesDatas->setHorizontalHeaderLabels (qsl);
    m_ui->twSurfacesDatas->setColumnCount(7);
    connect(m_ui->twSurfacesDatas,SIGNAL(cellChanged(int,int)),this,SLOT(OnCellChanged(int,int)));

    connect(m_ui->btnAddSurfaceAfter,SIGNAL(clicked()),this,SLOT(OnAddSurfaceAfter()));
    connect(m_ui->btnAddSurfaceBefore,SIGNAL(clicked()),this,SLOT(OnAddSurfaceBefore()));
    connect(m_ui->btnDeleteSurface,SIGNAL(clicked()),this,SLOT(OnDeleteSurface()));

    _bCanEmit=false;
}

DockSurfacesData::~DockSurfacesData()
{
    delete m_ui;
}

void DockSurfacesData::changeEvent(QEvent *e)
{
    QDockWidget::changeEvent(e);
    switch (e->type())
    {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}


void DockSurfacesData::device_changed(Device* pDevice)
{
    _bCanEmit=false;

    _pDevice=pDevice;
    m_ui->twSurfacesDatas->clearContents();
    m_ui->twSurfacesDatas->setRowCount(pDevice->nb_surface());

    for (int i=0;i<_pDevice->nb_surface();i++)
    {
        m_ui->twSurfacesDatas->setItem(i,0,new QTableWidgetItem(QString("%1").arg(i)));

        QComboBox*  qcbSurf=new QComboBox;
        qcbSurf->addItem("conic"); //TODO
        qcbSurf->addItem("perfect");
        int iPosGeometry=qcbSurf->findText(pDevice->get_geometry(i).c_str());
        assert(iPosGeometry!=-1);
        qcbSurf->setCurrentIndex(iPosGeometry);
        m_ui->twSurfacesDatas->setCellWidget(i,1,qcbSurf);
        connect(qcbSurf,SIGNAL(activated(int)),this,SLOT(onGeometryChanged()));

        QComboBox*  qcbKind=new QComboBox;
        qcbKind->addItem("stop"); //TODO
        qcbKind->addItem("reflect");
        qcbKind->addItem("transmit");
        qcbKind->addItem("observe");
        qcbKind->addItem("air");
        qcbKind->addItem("BK7");
        qcbKind->addItem("F2");
        int iPosKind=qcbKind->findText(pDevice->get_kind(i).c_str());
        assert(iPosKind!=-1);
        qcbKind->setCurrentIndex(iPosKind);
        m_ui->twSurfacesDatas->setCellWidget(i,2,qcbKind);
        connect(qcbKind,SIGNAL(activated(int)),this,SLOT(onKindChanged()));

        double dRC;
        pDevice->get(i,"radius_curvature",dRC);
        m_ui->twSurfacesDatas->setItem(i,3, new QTableWidgetItem(QString::number(dRC,'g',10)));

        double dConic;
        pDevice->get(i,"conic",dConic);
        m_ui->twSurfacesDatas->setItem(i,4, new QTableWidgetItem(QString::number(dConic,'g',10)));

        double dZ;
        pDevice->get(i,"z",dZ);
        m_ui->twSurfacesDatas->setItem(i,5,new QTableWidgetItem(QString::number(dZ,'g',10)));

        double dDiameter;
        pDevice->get(i,"diameter",dDiameter);
        m_ui->twSurfacesDatas->setItem(i,6,new QTableWidgetItem(QString::number(dDiameter,'g',10)));
    }

    _bCanEmit=true;

}
/////////////////////////////////////////////////////////////////////////://///
void DockSurfacesData::onKindChanged()
{
    for(int i=0;i<_pDevice->nb_surface();i++)
    {
        QComboBox* pComboBox=dynamic_cast<QComboBox*>(m_ui->twSurfacesDatas->cellWidget(i,2));
        assert(pComboBox!=0);
        if(pComboBox->currentText()!=_pDevice->get_kind(i).c_str())
            OnCellChanged(i,2);
    }
}
/////////////////////////////////////////////////////////////////////////://///
void DockSurfacesData::onGeometryChanged()
{
    for(int i=0;i<_pDevice->nb_surface();i++)
    {
        QComboBox* pComboBox=dynamic_cast<QComboBox*>(m_ui->twSurfacesDatas->cellWidget(i,1));
        assert(pComboBox!=0);
        if(pComboBox->currentText()!=_pDevice->get_geometry(i).c_str())
            OnCellChanged(i,1);
    }
}
/////////////////////////////////////////////////////////////////////////://///
void DockSurfacesData::OnCellChanged(int iRow,int iCol)
{
    if(_bCanEmit==false)
        return;

    if (iCol==1) //"geometry"
    {
        QComboBox* pComboBox=dynamic_cast<QComboBox*>(m_ui->twSurfacesDatas->cellWidget(iRow,1));
        assert(pComboBox!=0);
        QString qsText=pComboBox->currentText();
        bool bOk=_pDevice->set_geometry(iRow,qsText.toStdString()); (void)bOk; assert(bOk);
    }

    if (iCol==2) //"kind"
    {
        QComboBox* pComboBox=dynamic_cast<QComboBox*>(m_ui->twSurfacesDatas->cellWidget(iRow,2));
        assert(pComboBox!=0);
        QString qsText=pComboBox->currentText();
        bool bOk=_pDevice->set_kind(iRow,qsText.toStdString()); (void)bOk; assert(bOk);
    }

    if (iCol==3) //"radius_curvature"
    {
        QTableWidgetItem* pItem=m_ui->twSurfacesDatas->item(iRow,iCol);
        QString qsText=pItem->text();

        if (qsText=="inf")
            _pDevice->set(iRow,"curvature",0.);
        else
        {
            double d=qsText.toDouble();
            _pDevice->set(iRow,"radius_curvature",d);
        }
    }

    if (iCol==4) //"conic"
    {
        QTableWidgetItem* pItem=m_ui->twSurfacesDatas->item(iRow,iCol);
        QString qsText=pItem->text();

        double d=qsText.toDouble();
        _pDevice->set(iRow,"conic",d);
    }

    if (iCol==5) //"z"
    {
        QTableWidgetItem* pItem=m_ui->twSurfacesDatas->item(iRow,iCol);
        QString qsText=pItem->text();

        double d=qsText.toDouble();
        _pDevice->set(iRow,"z",d);
    }

    if (iCol==6) //"Diameter"
    {
        QTableWidgetItem* pItem=m_ui->twSurfacesDatas->item(iRow,iCol);
        QString qsText=pItem->text();

        double d=qsText.toDouble();
        _pDevice->set(iRow,"diameter",d);
    }

    if (_bCanEmit==true)
        emit device_changed(this);
}


void DockSurfacesData::OnAddSurfaceAfter()
{
    int iLine=m_ui->twSurfacesDatas->currentRow();
    if(iLine==-1)
    {
        if(_pDevice->nb_surface()!=0)
            return;
    }

    _pDevice->insert_surface("conic","stop",iLine+1);

    device_changed(_pDevice);
    emit device_changed(this);
}

void DockSurfacesData::OnAddSurfaceBefore()
{
    int iLine=m_ui->twSurfacesDatas->currentRow();
    if(iLine==-1)
    {
        if(_pDevice->nb_surface()!=0)
            return;
        iLine=0;
    }

    _pDevice->insert_surface("conic","stop",iLine);

    device_changed(_pDevice);
    emit device_changed(this);
}

void DockSurfacesData::OnDeleteSurface()
{
    int iLine=m_ui->twSurfacesDatas->currentRow();   
    if(iLine==-1)
        return;

    _pDevice->delete_surface(iLine);
    device_changed(_pDevice);
    emit device_changed(this);
}
