// this file is covered by the General Public License version 2 or later
// please see GPL.html for more details and licensing issues
// copyright Etienne de Foras ( the author )  mailto: etienne.deforas@gmail.com

#ifndef _Light_
#define _Light_

#include <string>
#include <vector>
using namespace std;

struct Photon;
class Medium;

class Light
{
public:
    Light(int iDimX=30,int iDimY=30);
    virtual ~Light();
    vector<Photon> get_all_photons();
    void set_nb_photons(int iNbX,int iNbY);

    Photon& get_photon(int iPos);
    int nb_photon();

    void set_tilt(double dTiltX,double dTiltY);
    void get_tilt(double& dTiltX,double& dTiltY);
    Medium& medium();
    void medium(Medium* pM);

    void set_colors(const string & vsAllColors);

    static void calc_colors(string sColors,vector<double>& vdLambda,vector<int>& vdVisualColors);

    void set_geometry(double dZ,double dDiameter);

    void calc_psf();
    void get_psf_center(double& dCenterX,double& dCenterY) const;
    double get_psf_diameter() const;
    double get_FD() const;
    double get_theorical_ro() const;
    double get_lfro() const;


private:
    void initialise();

    vector<Photon> _vPhotons;
    Medium* _pFirstMedium;
    Medium* _pMedium;
    bool _bPhotonsReady;
    double _dTiltX,_dTiltY;
    int _iDimX,_iDimY,_iNbPhotons;
    double _dDiameter,_dZ;
    double _dCenterX,_dCenterY,_dPsfDiameter;
    double _dFD;

    string _sAllColors;
};

#endif

/*
    // apres observe
    double vignetting();
    double FD();
    double lfro();
    void vignetting(double dVignetting);
    void FD(double dFD);
    void psf_size(double dPsfSize);
    bool focus(Light&l);
    double _dVignetting;
*/
