// this file is covered by the General Public License version 2 or later
// please see GPL.html for more details and licensing issues
// copyright Etienne de Foras ( the author )  mailto: etienne.deforas@gmail.com

#ifndef _Surface_
#define _Surface_

#include <string>
using namespace std;

class Light;
class Medium;
struct Photon;

class Surface
{
public:
    Surface();
    virtual ~Surface();

    string name();
    void name(string sName);

    string kind();
    bool set_kind(string sKind);

    string geometry();

    void z(double dz);
    double z();

    void diameter(double dDiameter);
    double diameter();

    virtual bool get(string sParam,double& dVal);
    virtual bool set(string sParam,double dVal);

    void receive(Light& l);

    double get_profile(double dX,double dY);

protected:
    bool verify_sq_radius(double d);
    void local_ref(Photon& p);
    void global_ref(Photon& p);

    virtual void stop(Light& l);
    virtual void reflect(Light& l);
    virtual void transmit(Light& l);
    virtual void observe(Light& l);

    virtual void stop_photon(Photon& p,bool bVerifyRadius=true)=0;
    virtual void reflect_photon(Photon& p)=0;
    virtual void transmit_photon(Photon& p)=0;

    Medium* _pMedium;
    Medium* _pMediumPrev;
    Medium* _pMediumNext;
    string _sGeometry;

private:
    string _sKind;

    double _z;
    double _dDiameter,_dDiameter2;
    bool _bDiameterAuto;
};

#endif
