// this file is covered by the General Public License version 2 or later
// please see GPL.html for more details and licensing issues
// copyright Etienne de Foras ( the author )  mailto: etienne.deforas@gmail.com

#include <iostream>
#include <cmath>
#include <cassert>

#include "SurfacePerfect.h"
#include "Photon.h"
#include "Medium.h"

///////////////////////////////////////////////////////////////////////////////
SurfacePerfect::SurfacePerfect()
{
    _dCurvature=0.;
    _sGeometry="perfect";
}
//////////////////////////////////////////////////////////////////////////////
bool SurfacePerfect::get(string sParam,double& dVal)
{
    if (sParam=="curvature")
    {
        dVal=_dCurvature;
        return true;
    }

    if (sParam=="conic")
    {
        dVal=0.;
        return true;
    }

    if (sParam=="radius_curvature")
    {
        dVal=1./_dCurvature;
        return true;
    }  // on espere !=0

    return Surface::get(sParam,dVal);
}
//////////////////////////////////////////////////////////////////////////////
bool SurfacePerfect::set(string sParam,double dVal)
{
    if (sParam=="curvature")
    {
        _dCurvature=dVal;
        return true;
    }

    if (sParam=="radius_curvature")
    {
        _dCurvature=1./dVal;
        return true;
    }

    return Surface::set(sParam,dVal);
}
///////////////////////////////////////////////////////////////////////////////
void SurfacePerfect::stop_photon(Photon& p,bool bVerifyRadius)
{
    if (p.is_valid()==false)
        return;

    if(p.dz==0.)
    {
        p.valid=false; // sait pas gerer
        return;
    }

    // arrete le photon en z=0
    double tmin=-p.z/p.dz;
    if (tmin<0.)
    {
        p.valid=false; // pas dans le futur du photon
        return;
    }

    p.x+=tmin*p.dx;
    p.y+=tmin*p.dy;
    p.z=0.;

    if (bVerifyRadius==false)
        return;

    double dRadiusSq=p.x*p.x+p.y*p.y;
    if ( verify_sq_radius(dRadiusSq))
        return;

    //pas d'intersection valable, on marque le point et on sort!
    p.valid=false;
}
///////////////////////////////////////////////////////////////////////////////
void SurfacePerfect::reflect_photon(Photon &p)
{
    if (!p.is_valid())
        return;

    transmit_photon(p);

    if (!p.is_valid())
        return;

    p.dz=-p.dz;
}
//////////////////////////////////////////////////////////////////////////////
void SurfacePerfect::transmit_photon(Photon& p)
{
    if (!p.valid)
        return;

    stop_photon(p);
    if (!p.valid)
        return;

    //calcul du point au foyer
    double az=0.5/_dCurvature;
    if(p.dz<0.)
           az=-az;

    double t=az/p.dz;
    double ax=p.dx*t;
    double ay=p.dy*t;

    //calcul du vecteur AB
    p.dx=ax-p.x;
    p.dy=ay-p.y;
    p.dz=az-p.z;

    if(_dCurvature<0.)
    {
        p.dx=-p.dx;
        p.dy=-p.dy;
        p.dz=-p.dz;
    }

    //normalise la direction
    double dDest=sqrt(p.dx*p.dx+p.dy*p.dy+p.dz*p.dz);
    p.dx/=dDest;
    p.dy/=dDest;
    p.dz/=dDest;
}
//////////////////////////////////////////////////////////////////////////////
